@@ -17,8 +17,11 @@
 #ifndef __GRE_H_
 #define __GRE_H_ 1
 
-#include "NetProto.h"
 #include "Flow.h"
+#include "IpHelper.h"
+#include "NetProto.h"
+
+typedef union _OVS_FWD_INFO *POVS_FWD_INFO;
 
 typedef struct _OVS_GRE_VPORT {
     UINT64 ipId;
@@ -66,7 +69,8 @@ NDIS_STATUS OvsEncapGre(POVS_VPORT_ENTRY vport,
                         OvsIPv4TunnelKey *tunKey,
                         POVS_SWITCH_CONTEXT switchContext,
                         POVS_PACKET_HDR_INFO layers,
-                        PNET_BUFFER_LIST *newNbl);
+                        PNET_BUFFER_LIST *newNbl,
+                        POVS_FWD_INFO switchFwdInfo);
 
 NDIS_STATUS OvsDecapGre(POVS_SWITCH_CONTEXT switchContext,
                         PNET_BUFFER_LIST curNbl,
