@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2011, 2012, 2013, 2014, 2016 Nicira, Inc.
+ * Copyright (c) 2010-2017 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -34,6 +34,7 @@ struct ofpact_stack;
 struct ofpbuf;
 struct nx_action_reg_load;
 struct nx_action_reg_move;
+struct vl_mff_map;
 
 
 /* Nicira Extended Match (NXM) flexible flow match helper functions.
@@ -76,17 +77,20 @@ int oxm_put_field_array(struct ofpbuf *, const struct field_array *,
 
 /* Decoding and encoding OXM/NXM headers (just a field ID) or entries (a field
  * ID followed by a value and possibly a mask). */
-enum ofperr nx_pull_entry(struct ofpbuf *, const struct mf_field **,
-                          union mf_value *value, union mf_value *mask);
-enum ofperr nx_pull_header(struct ofpbuf *, const struct mf_field **,
-                           bool *masked);
+enum ofperr nx_pull_entry(struct ofpbuf *, const struct vl_mff_map *,
+                          const struct mf_field **, union mf_value *value,
+                          union mf_value *mask);
+enum ofperr nx_pull_header(struct ofpbuf *, const struct vl_mff_map *,
+                           const struct mf_field **, bool *masked);
 void nxm_put__(struct ofpbuf *b, enum mf_field_id field,
                enum ofp_version version, const void *value,
                const void *mask, size_t n_bytes);
-void nx_put_entry(struct ofpbuf *, enum mf_field_id, enum ofp_version,
+void nx_put_entry(struct ofpbuf *, const struct mf_field *, enum ofp_version,
                   const union mf_value *value, const union mf_value *mask);
 void nx_put_header(struct ofpbuf *, enum mf_field_id, enum ofp_version,
                    bool masked);
+void nx_put_mff_header(struct ofpbuf *, const struct mf_field *,
+                       enum ofp_version, bool);
 
 /* NXM and OXM protocol headers values.
  *
@@ -95,7 +99,9 @@ void nx_put_header(struct ofpbuf *, enum mf_field_id, enum ofp_version,
  * the nx_*() functions should be preferred because they can support the 64-bit
  * "experimenter" OXM format (even though it is not yet implemented). */
 uint32_t mf_nxm_header(enum mf_field_id);
-const struct mf_field *mf_from_nxm_header(uint32_t nxm_header);
+uint32_t nxm_header_from_mff(const struct mf_field *);
+const struct mf_field *mf_from_nxm_header(uint32_t nxm_header,
+                                          const struct vl_mff_map *);
 
 char *nx_match_to_string(const uint8_t *, unsigned int match_len);
 char *oxm_match_to_string(const struct ofpbuf *, unsigned int match_len);
@@ -125,13 +131,16 @@ enum ofperr nxm_stack_push_check(const struct ofpact_stack *,
                                  const  struct flow *);
 enum ofperr nxm_stack_pop_check(const struct ofpact_stack *,
                                const struct flow *);
+void nx_stack_push(struct ofpbuf *stack, const void *v, uint8_t bytes);
+void nx_stack_push_bottom(struct ofpbuf *stack, const void *v, uint8_t bytes);
+void *nx_stack_pop(struct ofpbuf *stack, uint8_t *bytes);
 
 void nxm_execute_stack_push(const struct ofpact_stack *,
                             const struct flow *, struct flow_wildcards *,
                             struct ofpbuf *);
-void nxm_execute_stack_pop(const struct ofpact_stack *,
-                            struct flow *, struct flow_wildcards *,
-                            struct ofpbuf *);
+bool nxm_execute_stack_pop(const struct ofpact_stack *,
+                           struct flow *, struct flow_wildcards *,
+                           struct ofpbuf *);
 
 ovs_be64 oxm_bitmap_from_mf_bitmap(const struct mf_bitmap *, enum ofp_version);
 struct mf_bitmap oxm_bitmap_to_mf_bitmap(ovs_be64 oxm_bitmap,
