@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Nicira, Inc.
+ * Copyright (c) 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,74 +21,13 @@
 #include <inttypes.h>
 #include <limits.h>
 #include <stdarg.h>
-#include <stdbool.h>
-#include <stddef.h>
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "compiler.h"
-#include "openvswitch/types.h"
+#include "util.h"
 #include "openvswitch/util.h"
 
-#ifndef va_copy
-#ifdef __va_copy
-#define va_copy __va_copy
-#else
-#define va_copy(dst, src) ((dst) = (src))
-#endif
-#endif
-
-#ifdef __CHECKER__
-#define BUILD_ASSERT(EXPR) ((void) 0)
-#define BUILD_ASSERT_DECL(EXPR) extern int (*build_assert(void))[1]
-#elif !defined(__cplusplus)
-/* Build-time assertion building block. */
-#define BUILD_ASSERT__(EXPR) \
-        sizeof(struct { unsigned int build_assert_failed : (EXPR) ? 1 : -1; })
-
-/* Build-time assertion for use in a statement context. */
-#define BUILD_ASSERT(EXPR) (void) BUILD_ASSERT__(EXPR)
-
-/* Build-time assertion for use in a declaration context. */
-#define BUILD_ASSERT_DECL(EXPR) \
-        extern int (*build_assert(void))[BUILD_ASSERT__(EXPR)]
-#else /* __cplusplus */
-#include <boost/static_assert.hpp>
-#define BUILD_ASSERT BOOST_STATIC_ASSERT
-#define BUILD_ASSERT_DECL BOOST_STATIC_ASSERT
-#endif /* __cplusplus */
-
-#ifdef __GNUC__
-#define BUILD_ASSERT_GCCONLY(EXPR) BUILD_ASSERT(EXPR)
-#define BUILD_ASSERT_DECL_GCCONLY(EXPR) BUILD_ASSERT_DECL(EXPR)
-#else
-#define BUILD_ASSERT_GCCONLY(EXPR) ((void) 0)
-#define BUILD_ASSERT_DECL_GCCONLY(EXPR) ((void) 0)
-#endif
-
-/* Like the standard assert macro, except writes the failure message to the
- * log. */
-#ifndef NDEBUG
-#define ovs_assert(CONDITION)                                           \
-    if (!OVS_LIKELY(CONDITION)) {                                       \
-        ovs_assert_failure(OVS_SOURCE_LOCATOR, __func__, #CONDITION);       \
-    }
-#else
-#define ovs_assert(CONDITION) ((void) (CONDITION))
-#endif
-OVS_NO_RETURN void ovs_assert_failure(const char *, const char *, const char *);
-
-/* Casts 'pointer' to 'type' and issues a compiler warning if the cast changes
- * anything other than an outermost "const" or "volatile" qualifier.
- *
- * The cast to int is present only to suppress an "expression using sizeof
- * bool" warning from "sparse" (see
- * http://permalink.gmane.org/gmane.comp.parsers.sparse/2967). */
-#define CONST_CAST(TYPE, POINTER)                               \
-    ((void) sizeof ((int) ((POINTER) == (TYPE) (POINTER))),     \
-     (TYPE) (POINTER))
-
 extern char *program_name;
 
 #define __ARRAY_SIZE_NOCHECK(ARRAY) (sizeof(ARRAY) / sizeof((ARRAY)[0]))
@@ -106,48 +45,6 @@ extern char *program_name;
 #define __ARRAY_SIZE(ARRAY) __ARRAY_SIZE_NOCHECK(ARRAY)
 #endif
 
-/* Returns the number of elements in ARRAY. */
-#define ARRAY_SIZE(ARRAY) __ARRAY_SIZE(ARRAY)
-
-/* Returns X / Y, rounding up.  X must be nonnegative to round correctly. */
-#define DIV_ROUND_UP(X, Y) (((X) + ((Y) - 1)) / (Y))
-
-/* Returns X rounded up to the nearest multiple of Y. */
-#define ROUND_UP(X, Y) (DIV_ROUND_UP(X, Y) * (Y))
-
-/* Returns the least number that, when added to X, yields a multiple of Y. */
-#define PAD_SIZE(X, Y) (ROUND_UP(X, Y) - (X))
-
-/* Returns X rounded down to the nearest multiple of Y. */
-#define ROUND_DOWN(X, Y) ((X) / (Y) * (Y))
-
-/* Returns true if X is a power of 2, otherwise false. */
-#define IS_POW2(X) ((X) && !((X) & ((X) - 1)))
-
-static inline bool
-is_pow2(uintmax_t x)
-{
-    return IS_POW2(x);
-}
-
-/* Returns X rounded up to a power of 2.  X must be a constant expression. */
-#define ROUND_UP_POW2(X) RUP2__(X)
-#define RUP2__(X) (RUP2_1(X) + 1)
-#define RUP2_1(X) (RUP2_2(X) | (RUP2_2(X) >> 16))
-#define RUP2_2(X) (RUP2_3(X) | (RUP2_3(X) >> 8))
-#define RUP2_3(X) (RUP2_4(X) | (RUP2_4(X) >> 4))
-#define RUP2_4(X) (RUP2_5(X) | (RUP2_5(X) >> 2))
-#define RUP2_5(X) (RUP2_6(X) | (RUP2_6(X) >> 1))
-#define RUP2_6(X) ((X) - 1)
-
-/* Returns X rounded down to a power of 2.  X must be a constant expression. */
-#define ROUND_DOWN_POW2(X) RDP2__(X)
-#define RDP2__(X) (RDP2_1(X) - (RDP2_1(X) >> 1))
-#define RDP2_1(X) (RDP2_2(X) | (RDP2_2(X) >> 16))
-#define RDP2_2(X) (RDP2_3(X) | (RDP2_3(X) >> 8))
-#define RDP2_3(X) (RDP2_4(X) | (RDP2_4(X) >> 4))
-#define RDP2_4(X) (RDP2_5(X) | (RDP2_5(X) >> 2))
-#define RDP2_5(X) (      (X) | (      (X) >> 1))
 
 /* This system's cache line size, in bytes.
  * Being wrong hurts performance but not correctness. */
@@ -173,70 +70,17 @@ ovs_prefetch_range(const void *start, size_t size)
 #define MAX(X, Y) ((X) > (Y) ? (X) : (Y))
 #endif
 
+/* Comparisons for ints with modular arithmetic */
+#define INT_MOD_LT(a,b)     ((int) ((a)-(b)) < 0)
+#define INT_MOD_LEQ(a,b)    ((int) ((a)-(b)) <= 0)
+#define INT_MOD_GT(a,b)     ((int) ((a)-(b)) > 0)
+#define INT_MOD_GEQ(a,b)    ((int) ((a)-(b)) >= 0)
+
+#define INT_MOD_MIN(a, b)   ((INT_MOD_LT(a, b)) ? (a) : (b))
+#define INT_MOD_MAX(a, b)   ((INT_MOD_GT(a, b)) ? (a) : (b))
+
 #define OVS_NOT_REACHED() abort()
 
-/* Given a pointer-typed lvalue OBJECT, expands to a pointer type that may be
- * assigned to OBJECT. */
-#ifdef __GNUC__
-#define OVS_TYPEOF(OBJECT) typeof(OBJECT)
-#else
-#define OVS_TYPEOF(OBJECT) void *
-#endif
-
-/* Given OBJECT of type pointer-to-structure, expands to the offset of MEMBER
- * within an instance of the structure.
- *
- * The GCC-specific version avoids the technicality of undefined behavior if
- * OBJECT is null, invalid, or not yet initialized.  This makes some static
- * checkers (like Coverity) happier.  But the non-GCC version does not actually
- * dereference any pointer, so it would be surprising for it to cause any
- * problems in practice.
- */
-#ifdef __GNUC__
-#define OBJECT_OFFSETOF(OBJECT, MEMBER) offsetof(typeof(*(OBJECT)), MEMBER)
-#else
-#define OBJECT_OFFSETOF(OBJECT, MEMBER) \
-    ((char *) &(OBJECT)->MEMBER - (char *) (OBJECT))
-#endif
-
-/* Yields the size of MEMBER within STRUCT. */
-#define MEMBER_SIZEOF(STRUCT, MEMBER) (sizeof(((STRUCT *) NULL)->MEMBER))
-
-/* Given POINTER, the address of the given MEMBER in a STRUCT object, returns
-   the STRUCT object. */
-#define CONTAINER_OF(POINTER, STRUCT, MEMBER)                           \
-        ((STRUCT *) (void *) ((char *) (POINTER) - offsetof (STRUCT, MEMBER)))
-
-/* Given POINTER, the address of the given MEMBER within an object of the type
- * that that OBJECT points to, returns OBJECT as an assignment-compatible
- * pointer type (either the correct pointer type or "void *").  OBJECT must be
- * an lvalue.
- *
- * This is the same as CONTAINER_OF except that it infers the structure type
- * from the type of '*OBJECT'. */
-#define OBJECT_CONTAINING(POINTER, OBJECT, MEMBER)                      \
-    ((OVS_TYPEOF(OBJECT)) (void *)                                      \
-     ((char *) (POINTER) - OBJECT_OFFSETOF(OBJECT, MEMBER)))
-
-/* Given POINTER, the address of the given MEMBER within an object of the type
- * that that OBJECT points to, assigns the address of the outer object to
- * OBJECT, which must be an lvalue.
- *
- * Evaluates to (void) 0 as the result is not to be used. */
-#define ASSIGN_CONTAINER(OBJECT, POINTER, MEMBER) \
-    ((OBJECT) = OBJECT_CONTAINING(POINTER, OBJECT, MEMBER), (void) 0)
-
-/* As explained in the comment above OBJECT_OFFSETOF(), non-GNUC compilers
- * like MSVC will complain about un-initialized variables if OBJECT
- * hasn't already been initialized. To prevent such warnings, INIT_CONTAINER()
- * can be used as a wrapper around ASSIGN_CONTAINER. */
-#define INIT_CONTAINER(OBJECT, POINTER, MEMBER) \
-    ((OBJECT) = NULL, ASSIGN_CONTAINER(OBJECT, POINTER, MEMBER))
-
-/* Given ATTR, and TYPE, cast the ATTR to TYPE by first casting ATTR to
- * (void *). This is to suppress the alignment warning issued by clang. */
-#define ALIGNED_CAST(TYPE, ATTR) ((TYPE) (void *) (ATTR))
-
 /* Use "%"PRIuSIZE to format size_t with printf(). */
 #ifdef _WIN32
 #define PRIdSIZE "Id"
@@ -278,6 +122,8 @@ void *xrealloc(void *, size_t);
 void *xmemdup(const void *, size_t) MALLOC_LIKE;
 char *xmemdup0(const char *, size_t) MALLOC_LIKE;
 char *xstrdup(const char *) MALLOC_LIKE;
+char *nullable_xstrdup(const char *) MALLOC_LIKE;
+bool nullable_string_is_equal(const char *a, const char *b);
 char *xasprintf(const char *format, ...) OVS_PRINTF_FORMAT(1, 2) MALLOC_LIKE;
 char *xvasprintf(const char *format, va_list) OVS_PRINTF_FORMAT(1, 0) MALLOC_LIKE;
 void *x2nrealloc(void *p, size_t *n, size_t s);
@@ -307,7 +153,9 @@ void ovs_hex_dump(FILE *, const void *, size_t, uintptr_t offset, bool ascii);
 bool str_to_int(const char *, int base, int *);
 bool str_to_long(const char *, int base, long *);
 bool str_to_llong(const char *, int base, long long *);
+bool str_to_llong_with_tail(const char *, char **, int base, long long *);
 bool str_to_uint(const char *, int base, unsigned int *);
+bool str_to_llong_range(const char *, int base, long long *, long long *);
 
 bool ovs_scan(const char *s, const char *format, ...) OVS_SCANF_FORMAT(2, 3);
 bool ovs_scan_len(const char *s, int *n, const char *format, ...);
@@ -558,37 +406,37 @@ void bitwise_toggle_bit(void *dst, unsigned int len, unsigned int ofs);
 
 /* Returns non-zero if the parameters have equal value. */
 static inline int
-ovs_u128_equals(const ovs_u128 *a, const ovs_u128 *b)
+ovs_u128_equals(const ovs_u128 a, const ovs_u128 b)
 {
-    return (a->u64.hi == b->u64.hi) && (a->u64.lo == b->u64.lo);
+    return (a.u64.hi == b.u64.hi) && (a.u64.lo == b.u64.lo);
 }
 
 /* Returns true if 'val' is 0. */
 static inline bool
-ovs_u128_is_zero(const ovs_u128 *val)
+ovs_u128_is_zero(const ovs_u128 val)
 {
-    return !(val->u64.hi || val->u64.lo);
+    return !(val.u64.hi || val.u64.lo);
 }
 
 /* Returns true if 'val' is all ones. */
 static inline bool
-ovs_u128_is_ones(const ovs_u128 *val)
+ovs_u128_is_ones(const ovs_u128 val)
 {
-    return ovs_u128_equals(val, &OVS_U128_MAX);
+    return ovs_u128_equals(val, OVS_U128_MAX);
 }
 
 /* Returns non-zero if the parameters have equal value. */
 static inline int
-ovs_be128_equals(const ovs_be128 *a, const ovs_be128 *b)
+ovs_be128_equals(const ovs_be128 a, const ovs_be128 b)
 {
-    return (a->be64.hi == b->be64.hi) && (a->be64.lo == b->be64.lo);
+    return (a.be64.hi == b.be64.hi) && (a.be64.lo == b.be64.lo);
 }
 
 /* Returns true if 'val' is 0. */
 static inline bool
-ovs_be128_is_zero(const ovs_be128 *val)
+ovs_be128_is_zero(const ovs_be128 val)
 {
-    return !(val->be64.hi || val->be64.lo);
+    return !(val.be64.hi || val.be64.lo);
 }
 
 static inline ovs_u128
@@ -604,6 +452,8 @@ ovs_u128_and(const ovs_u128 a, const ovs_u128 b)
 
 void xsleep(unsigned int seconds);
 
+bool is_stdout_a_tty(void);
+
 #ifdef _WIN32
 
 char *ovs_format_message(int error);
