@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016 Nicira, Inc.
+ * Copyright (c) 2009-2017 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -52,6 +52,7 @@
 #include "timeval.h"
 #include "tun-metadata.h"
 #include "versions.h"
+#include "vl-mff-map.h"
 
 struct match;
 struct ofputil_flow_mod;
@@ -126,6 +127,10 @@ struct ofproto {
 
      /* Tunnel TLV mapping table. */
      OVSRCU_TYPE(struct tun_table *) metadata_tab;
+
+    /* Variable length mf_field mapping. Stores all configured variable length
+     * meta-flow fields (struct mf_field) in a switch. */
+    struct vl_mff_map vl_mff_map;
 };
 
 void ofproto_init_tables(struct ofproto *, int n_tables);
@@ -365,8 +370,8 @@ struct rule {
     struct ovs_refcount ref_count;
 
     /* A "flow cookie" is the OpenFlow name for a 64-bit value associated with
-     * a flow.. */
-    ovs_be64 flow_cookie OVS_GUARDED;
+     * a flow. */
+    const ovs_be64 flow_cookie; /* Immutable once rule is constructed. */
     struct hindex_node cookie_node OVS_GUARDED_BY(ofproto_mutex);
 
     enum ofputil_flow_mod_flags flags OVS_GUARDED;
