@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2016 Nicira, Inc.
+/* Copyright (c) 2015, 2016, 2017 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -43,20 +43,6 @@ lflow_init(void)
 {
     ovn_init_symtab(&symtab);
 }
-
-/* Iterate address sets in the southbound database.  Create and update the
- * corresponding symtab entries as necessary. */
-static void
-update_address_sets(struct controller_ctx *ctx,
-                    struct shash *expr_address_sets_p)
-
-{
-    const struct sbrec_address_set *as;
-    SBREC_ADDRESS_SET_FOR_EACH (as, ctx->ovnsb_idl) {
-        expr_macros_add(expr_address_sets_p, as->name,
-                        (const char *const *) as->addresses, as->n_addresses);
-    }
-}
 
 struct lookup_port_aux {
     const struct lport_index *lports;
@@ -64,17 +50,23 @@ struct lookup_port_aux {
     const struct sbrec_datapath_binding *dp;
 };
 
+struct condition_aux {
+    const struct lport_index *lports;
+    const struct sbrec_chassis *chassis;
+};
+
 static void consider_logical_flow(const struct lport_index *lports,
                                   const struct mcgroup_index *mcgroups,
                                   const struct sbrec_logical_flow *lflow,
                                   const struct hmap *local_datapaths,
                                   struct group_table *group_table,
                                   const struct simap *ct_zones,
-                                  struct hmap *dhcp_opts_p,
-                                  struct hmap *dhcpv6_opts_p,
-                                  uint32_t *conj_id_ofs_p,
-                                  struct hmap *flow_table,
-                                  struct shash *expr_address_sets_p);
+                                  const struct sbrec_chassis *chassis,
+                                  struct hmap *dhcp_opts,
+                                  struct hmap *dhcpv6_opts,
+                                  uint32_t *conj_id_ofs,
+                                  const struct shash *addr_sets,
+                                  struct hmap *flow_table);
 
 static bool
 lookup_port_cb(const void *aux_, const char *port_name, unsigned int *portp)
@@ -98,6 +90,16 @@ lookup_port_cb(const void *aux_, const char *port_name, unsigned int *portp)
     return false;
 }
 
+static bool
+is_chassis_resident_cb(const void *c_aux_, const char *port_name)
+{
+    const struct condition_aux *c_aux = c_aux_;
+
+    const struct sbrec_port_binding *pb
+        = lport_lookup_by_name(c_aux->lports, port_name);
+    return pb && pb->chassis && pb->chassis == c_aux->chassis;
+}
+
 static bool
 is_switch(const struct sbrec_datapath_binding *ldp)
 {
@@ -105,6 +107,15 @@ is_switch(const struct sbrec_datapath_binding *ldp)
 
 }
 
+static bool
+is_gateway_router(const struct sbrec_datapath_binding *ldp,
+                  const struct hmap *local_datapaths)
+{
+    struct local_datapath *ld =
+        get_local_datapath(local_datapaths, ldp->tunnel_key);
+    return ld ? ld->has_local_l3gateway : false;
+}
+
 /* Adds the logical flows from the Logical_Flow table to flow tables. */
 static void
 add_logical_flows(struct controller_ctx *ctx, const struct lport_index *lports,
@@ -112,8 +123,9 @@ add_logical_flows(struct controller_ctx *ctx, const struct lport_index *lports,
                   const struct hmap *local_datapaths,
                   struct group_table *group_table,
                   const struct simap *ct_zones,
-                  struct hmap *flow_table,
-                  struct shash *expr_address_sets_p)
+                  const struct sbrec_chassis *chassis,
+                  const struct shash *addr_sets,
+                  struct hmap *flow_table)
 {
     uint32_t conj_id_ofs = 1;
     const struct sbrec_logical_flow *lflow;
@@ -135,9 +147,9 @@ add_logical_flows(struct controller_ctx *ctx, const struct lport_index *lports,
 
     SBREC_LOGICAL_FLOW_FOR_EACH (lflow, ctx->ovnsb_idl) {
         consider_logical_flow(lports, mcgroups, lflow, local_datapaths,
-                              group_table, ct_zones,
+                              group_table, ct_zones, chassis,
                               &dhcp_opts, &dhcpv6_opts, &conj_id_ofs,
-                              flow_table, expr_address_sets_p);
+                              addr_sets, flow_table);
     }
 
     dhcp_opts_destroy(&dhcp_opts);
@@ -151,11 +163,12 @@ consider_logical_flow(const struct lport_index *lports,
                       const struct hmap *local_datapaths,
                       struct group_table *group_table,
                       const struct simap *ct_zones,
-                      struct hmap *dhcp_opts_p,
-                      struct hmap *dhcpv6_opts_p,
-                      uint32_t *conj_id_ofs_p,
-                      struct hmap *flow_table,
-                      struct shash *expr_address_sets_p)
+                      const struct sbrec_chassis *chassis,
+                      struct hmap *dhcp_opts,
+                      struct hmap *dhcpv6_opts,
+                      uint32_t *conj_id_ofs,
+                      const struct shash *addr_sets,
+                      struct hmap *flow_table)
 {
     /* Determine translation of logical table IDs to physical table IDs. */
     bool ingress = !strcmp(lflow->pipeline, "ingress");
@@ -184,9 +197,10 @@ consider_logical_flow(const struct lport_index *lports,
     struct ofpbuf ovnacts = OFPBUF_STUB_INITIALIZER(ovnacts_stub);
     struct ovnact_parse_params pp = {
         .symtab = &symtab,
-        .dhcp_opts = dhcp_opts_p,
-        .dhcpv6_opts = dhcpv6_opts_p,
+        .dhcp_opts = dhcp_opts,
+        .dhcpv6_opts = dhcpv6_opts,
 
+        .pipeline = ingress ? OVNACT_P_INGRESS : OVNACT_P_EGRESS,
         .n_tables = LOG_PIPELINE_LEN,
         .cur_ltable = lflow->table_id,
     };
@@ -216,10 +230,13 @@ consider_logical_flow(const struct lport_index *lports,
         .lookup_port = lookup_port_cb,
         .aux = &aux,
         .is_switch = is_switch(ldp),
+        .is_gateway_router = is_gateway_router(ldp, local_datapaths),
         .ct_zones = ct_zones,
         .group_table = group_table,
 
-        .first_ptable = first_ptable,
+        .pipeline = ingress ? OVNACT_P_INGRESS : OVNACT_P_EGRESS,
+        .ingress_ptable = OFTABLE_LOG_INGRESS_PIPELINE,
+        .egress_ptable = OFTABLE_LOG_EGRESS_PIPELINE,
         .output_ptable = output_ptable,
         .mac_bind_ptable = OFTABLE_MAC_BINDING,
     };
@@ -231,8 +248,7 @@ consider_logical_flow(const struct lport_index *lports,
     struct hmap matches;
     struct expr *expr;
 
-    expr = expr_parse_string(lflow->match, &symtab,
-                             expr_address_sets_p, &error);
+    expr = expr_parse_string(lflow->match, &symtab, addr_sets, &error);
     if (!error) {
         if (prereqs) {
             expr = expr_combine(EXPR_T_AND, expr, prereqs);
@@ -250,7 +266,8 @@ consider_logical_flow(const struct lport_index *lports,
         return;
     }
 
-    expr = expr_simplify(expr);
+    struct condition_aux cond_aux = { lports, chassis };
+    expr = expr_simplify(expr, is_chassis_resident_cb, &cond_aux);
     expr = expr_normalize(expr);
     uint32_t n_conjs = expr_to_matches(expr, lookup_port_cb, &aux,
                                        &matches);
@@ -262,11 +279,11 @@ consider_logical_flow(const struct lport_index *lports,
         match_set_metadata(&m->match,
                            htonll(lflow->logical_datapath->tunnel_key));
         if (m->match.wc.masks.conj_id) {
-            m->match.flow.conj_id += *conj_id_ofs_p;
+            m->match.flow.conj_id += *conj_id_ofs;
         }
         if (!m->n) {
-            ofctrl_add_flow(flow_table, ptable, lflow->priority, &m->match,
-                            &ofpacts);
+            ofctrl_add_flow(flow_table, ptable, lflow->priority,
+                            lflow->header_.uuid.parts[0], &m->match, &ofpacts);
         } else {
             uint64_t conj_stubs[64 / 8];
             struct ofpbuf conj;
@@ -277,11 +294,11 @@ consider_logical_flow(const struct lport_index *lports,
                 struct ofpact_conjunction *dst;
 
                 dst = ofpact_put_CONJUNCTION(&conj);
-                dst->id = src->id + *conj_id_ofs_p;
+                dst->id = src->id + *conj_id_ofs;
                 dst->clause = src->clause;
                 dst->n_clauses = src->n_clauses;
             }
-            ofctrl_add_flow(flow_table, ptable, lflow->priority, &m->match,
+            ofctrl_add_flow(flow_table, ptable, lflow->priority, 0, &m->match,
                             &conj);
             ofpbuf_uninit(&conj);
         }
@@ -290,7 +307,7 @@ consider_logical_flow(const struct lport_index *lports,
     /* Clean up. */
     expr_matches_destroy(&matches);
     ofpbuf_uninit(&ofpacts);
-    *conj_id_ofs_p += n_conjs;
+    *conj_id_ofs += n_conjs;
 }
 
 static void
@@ -350,7 +367,7 @@ consider_neighbor_flow(const struct lport_index *lports,
     uint64_t stub[1024 / 8];
     struct ofpbuf ofpacts = OFPBUF_STUB_INITIALIZER(stub);
     put_load(mac.ea, sizeof mac.ea, MFF_ETH_DST, 0, 48, &ofpacts);
-    ofctrl_add_flow(flow_table, OFTABLE_MAC_BINDING, 100, &match, &ofpacts);
+    ofctrl_add_flow(flow_table, OFTABLE_MAC_BINDING, 100, 0, &match, &ofpacts);
     ofpbuf_uninit(&ofpacts);
 }
 
@@ -371,22 +388,19 @@ add_neighbor_flows(struct controller_ctx *ctx,
 /* Translates logical flows in the Logical_Flow table in the OVN_SB database
  * into OpenFlow flows.  See ovn-architecture(7) for more information. */
 void
-lflow_run(struct controller_ctx *ctx, const struct lport_index *lports,
+lflow_run(struct controller_ctx *ctx,
+          const struct sbrec_chassis *chassis,
+          const struct lport_index *lports,
           const struct mcgroup_index *mcgroups,
           const struct hmap *local_datapaths,
           struct group_table *group_table,
           const struct simap *ct_zones,
+          const struct shash *addr_sets,
           struct hmap *flow_table)
 {
-    struct shash expr_address_sets = SHASH_INITIALIZER(&expr_address_sets);
-
-    update_address_sets(ctx, &expr_address_sets);
-    add_logical_flows(ctx, lports, mcgroups, local_datapaths,
-                      group_table, ct_zones, flow_table, &expr_address_sets);
+    add_logical_flows(ctx, lports, mcgroups, local_datapaths, group_table,
+                      ct_zones, chassis, addr_sets, flow_table);
     add_neighbor_flows(ctx, lports, flow_table);
-
-    expr_macros_destroy(&expr_address_sets);
-    shash_destroy(&expr_address_sets);
 }
 
 void
