@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2016 Nicira, Inc.
+ * Copyright (c) 2015, 2016, 2017 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,7 +45,7 @@ VLOG_DEFINE_THIS_MODULE(actions);
     static void encode_##ENUM(const struct STRUCT *,                \
                               const struct ovnact_encode_params *,  \
                               struct ofpbuf *ofpacts);              \
-    static void free_##ENUM(struct STRUCT *a);
+    static void STRUCT##_free(struct STRUCT *a);
 OVNACTS
 #undef OVNACT
 
@@ -55,10 +55,10 @@ OVNACTS
 void *
 ovnact_put(struct ofpbuf *ovnacts, enum ovnact_type type, size_t len)
 {
-    struct ovnact *ovnact;
+    ovs_assert(len == OVNACT_ALIGN(len));
 
     ovnacts->header = ofpbuf_put_uninit(ovnacts, len);
-    ovnact = ovnacts->header;
+    struct ovnact *ovnact = ovnacts->header;
     ovnact_init(ovnact, type, len);
     return ovnact;
 }
@@ -67,6 +67,7 @@ ovnact_put(struct ofpbuf *ovnacts, enum ovnact_type type, size_t len)
 void
 ovnact_init(struct ovnact *ovnact, enum ovnact_type type, size_t len)
 {
+    ovs_assert(len == OVNACT_ALIGN(len));
     memset(ovnact, 0, len);
     ovnact->type = type;
     ovnact->len = len;
@@ -169,6 +170,15 @@ put_load(uint64_t value, enum mf_field_id dst, int ofs, int n_bits,
     bitwise_copy(&n_value, 8, 0, sf->value, sf->field->n_bytes, ofs, n_bits);
     bitwise_one(ofpact_set_field_mask(sf), sf->field->n_bytes, ofs, n_bits);
 }
+
+static uint8_t
+first_ptable(const struct ovnact_encode_params *ep,
+             enum ovnact_pipeline pipeline)
+{
+    return (pipeline == OVNACT_P_INGRESS
+            ? ep->ingress_ptable
+            : ep->egress_ptable);
+}
 
 /* Context maintained during ovnacts_parse(). */
 struct action_context {
@@ -178,7 +188,7 @@ struct action_context {
     struct expr *prereqs;       /* Prerequisites to apply to match. */
 };
 
-static bool parse_action(struct action_context *);
+static void parse_actions(struct action_context *, enum lex_type sentinel);
 
 static bool
 action_parse_field(struct action_context *ctx,
@@ -225,6 +235,11 @@ add_prerequisite(struct action_context *ctx, const char *prerequisite)
     ovs_assert(!error);
     ctx->prereqs = expr_combine(EXPR_T_AND, ctx->prereqs, expr);
 }
+
+static void
+ovnact_null_free(struct ovnact_null *a OVS_UNUSED)
+{
+}
 
 static void
 format_OUTPUT(const struct ovnact_null *a OVS_UNUSED, struct ds *s)
@@ -247,47 +262,83 @@ encode_OUTPUT(const struct ovnact_null *a OVS_UNUSED,
 {
     emit_resubmit(ofpacts, ep->output_ptable);
 }
-
-static void
-free_OUTPUT(struct ovnact_null *a OVS_UNUSED)
-{
-}
 
 static void
 parse_NEXT(struct action_context *ctx)
 {
     if (!ctx->pp->n_tables) {
         lexer_error(ctx->lexer, "\"next\" action not allowed here.");
-    } else if (lexer_match(ctx->lexer, LEX_T_LPAREN)) {
-        int ltable;
+        return;
+    }
 
-        if (!lexer_force_int(ctx->lexer, &ltable) ||
-            !lexer_force_match(ctx->lexer, LEX_T_RPAREN)) {
-            return;
-        }
-
-        if (ltable >= ctx->pp->n_tables) {
-            lexer_error(ctx->lexer,
-                        "\"next\" argument must be in range 0 to %d.",
-                         ctx->pp->n_tables - 1);
-            return;
-        }
-
-        ovnact_put_NEXT(ctx->ovnacts)->ltable = ltable;
-    } else {
-        if (ctx->pp->cur_ltable < ctx->pp->n_tables) {
-            ovnact_put_NEXT(ctx->ovnacts)->ltable = ctx->pp->cur_ltable + 1;
+    int pipeline = ctx->pp->pipeline;
+    int table = ctx->pp->cur_ltable + 1;
+    if (lexer_match(ctx->lexer, LEX_T_LPAREN)) {
+        if (lexer_is_int(ctx->lexer)) {
+            lexer_get_int(ctx->lexer, &table);
         } else {
-            lexer_error(ctx->lexer,
-                        "\"next\" action not allowed in last table.");
+            do {
+                if (lexer_match_id(ctx->lexer, "pipeline")) {
+                    if (!lexer_force_match(ctx->lexer, LEX_T_EQUALS)) {
+                        return;
+                    }
+                    if (lexer_match_id(ctx->lexer, "ingress")) {
+                        pipeline = OVNACT_P_INGRESS;
+                    } else if (lexer_match_id(ctx->lexer, "egress")) {
+                        pipeline = OVNACT_P_EGRESS;
+                    } else {
+                        lexer_syntax_error(
+                            ctx->lexer, "expecting \"ingress\" or \"egress\"");
+                        return;
+                    }
+                } else if (lexer_match_id(ctx->lexer, "table")) {
+                    if (!lexer_force_match(ctx->lexer, LEX_T_EQUALS) ||
+                        !lexer_force_int(ctx->lexer, &table)) {
+                        return;
+                    }
+                } else {
+                    lexer_syntax_error(ctx->lexer,
+                                       "expecting \"pipeline\" or \"table\"");
+                    return;
+                }
+            } while (lexer_match(ctx->lexer, LEX_T_COMMA));
+        }
+        if (!lexer_force_match(ctx->lexer, LEX_T_RPAREN)) {
+            return;
         }
     }
+
+    if (pipeline == OVNACT_P_EGRESS && ctx->pp->pipeline == OVNACT_P_INGRESS) {
+        lexer_error(ctx->lexer,
+                    "\"next\" action cannot advance from ingress to egress "
+                    "pipeline (use \"output\" action instead)");
+    } else if (table >= ctx->pp->n_tables) {
+        lexer_error(ctx->lexer,
+                    "\"next\" action cannot advance beyond table %d.",
+                    ctx->pp->n_tables - 1);
+        return;
+    }
+
+    struct ovnact_next *next = ovnact_put_NEXT(ctx->ovnacts);
+    next->pipeline = pipeline;
+    next->ltable = table;
+    next->src_pipeline = ctx->pp->pipeline;
+    next->src_ltable = ctx->pp->cur_ltable;
 }
 
 static void
 format_NEXT(const struct ovnact_next *next, struct ds *s)
 {
-    ds_put_format(s, "next(%d);", next->ltable);
+    if (next->pipeline != next->src_pipeline) {
+        ds_put_format(s, "next(pipeline=%s, table=%d);",
+                      (next->pipeline == OVNACT_P_INGRESS
+                       ? "ingress" : "egress"),
+                      next->ltable);
+    } else if (next->ltable != next->src_ltable + 1) {
+        ds_put_format(s, "next(%d);", next->ltable);
+    } else {
+        ds_put_cstr(s, "next;");
+    }
 }
 
 static void
@@ -295,11 +346,11 @@ encode_NEXT(const struct ovnact_next *next,
             const struct ovnact_encode_params *ep,
             struct ofpbuf *ofpacts)
 {
-    emit_resubmit(ofpacts, ep->first_ptable + next->ltable);
+    emit_resubmit(ofpacts, first_ptable(ep, next->pipeline) + next->ltable);
 }
 
 static void
-free_NEXT(struct ovnact_next *a OVS_UNUSED)
+ovnact_next_free(struct ovnact_next *a OVS_UNUSED)
 {
 }
 
@@ -373,7 +424,7 @@ encode_LOAD(const struct ovnact_load *load,
 }
 
 static void
-free_LOAD(struct ovnact_load *load)
+ovnact_load_free(struct ovnact_load *load)
 {
     expr_constant_destroy(&load->imm, load_type(load));
 }
@@ -489,12 +540,7 @@ encode_EXCHANGE(const struct ovnact_move *xchg,
 }
 
 static void
-free_MOVE(struct ovnact_move *move OVS_UNUSED)
-{
-}
-
-static void
-free_EXCHANGE(struct ovnact_move *xchg OVS_UNUSED)
+ovnact_move_free(struct ovnact_move *move OVS_UNUSED)
 {
 }
 
@@ -519,11 +565,6 @@ encode_DEC_TTL(const struct ovnact_null *null OVS_UNUSED,
 {
     ofpact_put_DEC_TTL(ofpacts);
 }
-
-static void
-free_DEC_TTL(struct ovnact_null *null OVS_UNUSED)
-{
-}
 
 static void
 parse_CT_NEXT(struct action_context *ctx)
@@ -539,18 +580,18 @@ parse_CT_NEXT(struct action_context *ctx)
 }
 
 static void
-format_CT_NEXT(const struct ovnact_next *next OVS_UNUSED, struct ds *s)
+format_CT_NEXT(const struct ovnact_ct_next *ct_next OVS_UNUSED, struct ds *s)
 {
     ds_put_cstr(s, "ct_next;");
 }
 
 static void
-encode_CT_NEXT(const struct ovnact_next *next,
+encode_CT_NEXT(const struct ovnact_ct_next *ct_next,
                 const struct ovnact_encode_params *ep,
                 struct ofpbuf *ofpacts)
 {
     struct ofpact_conntrack *ct = ofpact_put_CT(ofpacts);
-    ct->recirc_table = ep->first_ptable + next->ltable;
+    ct->recirc_table = first_ptable(ep, ep->pipeline) + ct_next->ltable;
     ct->zone_src.field = ep->is_switch ? mf_from_id(MFF_LOG_CT_ZONE)
                             : mf_from_id(MFF_LOG_DNAT_ZONE);
     ct->zone_src.ofs = 0;
@@ -559,7 +600,7 @@ encode_CT_NEXT(const struct ovnact_next *next,
 }
 
 static void
-free_CT_NEXT(struct ovnact_next *next OVS_UNUSED)
+ovnact_ct_next_free(struct ovnact_ct_next *a OVS_UNUSED)
 {
 }
 
@@ -679,7 +720,7 @@ encode_CT_COMMIT(const struct ovnact_ct_commit *cc,
 }
 
 static void
-free_CT_COMMIT(struct ovnact_ct_commit *cc OVS_UNUSED)
+ovnact_ct_commit_free(struct ovnact_ct_commit *cc OVS_UNUSED)
 {
 }
 
@@ -754,7 +795,7 @@ encode_ct_nat(const struct ovnact_ct_nat *cn,
     ofpbuf_pull(ofpacts, ct_offset);
 
     struct ofpact_conntrack *ct = ofpact_put_CT(ofpacts);
-    ct->recirc_table = cn->ltable + ep->first_ptable;
+    ct->recirc_table = cn->ltable + first_ptable(ep, ep->pipeline);
     if (snat) {
         ct->zone_src.field = mf_from_id(MFF_LOG_SNAT_ZONE);
     } else {
@@ -788,12 +829,15 @@ encode_ct_nat(const struct ovnact_ct_nat *cn,
     ct = ofpacts->header;
     if (cn->ip) {
         ct->flags |= NX_CT_F_COMMIT;
-    } else if (snat) {
-        /* XXX: For performance reasons, we try to prevent additional
-         * recirculations.  So far, ct_snat which is used in a gateway router
-         * does not need a recirculation. ct_snat(IP) does need a
-         * recirculation.  Should we consider a method to let the actions
-         * specify whether an action needs recirculation if there more use
+    } else if (snat && ep->is_gateway_router) {
+        /* For performance reasons, we try to prevent additional
+         * recirculations.  ct_snat which is used in a gateway router
+         * does not need a recirculation.  ct_snat(IP) does need a
+         * recirculation.  ct_snat in a distributed router needs
+         * recirculation regardless of whether an IP address is
+         * specified.
+         * XXX Should we consider a method to let the actions specify
+         * whether an action needs recirculation if there are more use
          * cases?. */
         ct->recirc_table = NX_CT_RECIRC_NONE;
     }
@@ -818,12 +862,7 @@ encode_CT_SNAT(const struct ovnact_ct_nat *cn,
 }
 
 static void
-free_CT_DNAT(struct ovnact_ct_nat *ct_nat OVS_UNUSED)
-{
-}
-
-static void
-free_CT_SNAT(struct ovnact_ct_nat *ct_nat OVS_UNUSED)
+ovnact_ct_nat_free(struct ovnact_ct_nat *ct_nat OVS_UNUSED)
 {
 }
 
@@ -903,7 +942,7 @@ encode_CT_LB(const struct ovnact_ct_lb *cl,
              const struct ovnact_encode_params *ep,
              struct ofpbuf *ofpacts)
 {
-    uint8_t recirc_table = cl->ltable + ep->first_ptable;
+    uint8_t recirc_table = cl->ltable + first_ptable(ep, ep->pipeline);
     if (!cl->n_dsts) {
         /* ct_lb without any destinations means that this is an established
          * connection and we just need to do a NAT. */
@@ -1015,13 +1054,27 @@ encode_CT_LB(const struct ovnact_ct_lb *cl,
 }
 
 static void
-free_CT_LB(struct ovnact_ct_lb *ct_lb)
+ovnact_ct_lb_free(struct ovnact_ct_lb *ct_lb)
 {
     free(ct_lb->dsts);
 }
 
-/* Implements the "arp" and "nd_na" actions, which execute nested actions on a
- * packet derived from the one being processed. */
+static void
+format_CT_CLEAR(const struct ovnact_null *null OVS_UNUSED, struct ds *s)
+{
+    ds_put_cstr(s, "ct_clear;");
+}
+
+static void
+encode_CT_CLEAR(const struct ovnact_null *null OVS_UNUSED,
+                const struct ovnact_encode_params *ep OVS_UNUSED,
+                struct ofpbuf *ofpacts)
+{
+    ofpact_put_CT_CLEAR(ofpacts);
+}
+
+/* Implements the "arp", "nd_na", and "clone" actions, which execute nested
+ * actions on a packet derived from the one being processed. */
 static void
 parse_nested_action(struct action_context *ctx, enum ovnact_type type,
                     const char *prereq)
@@ -1037,17 +1090,21 @@ parse_nested_action(struct action_context *ctx, enum ovnact_type type,
         .pp = ctx->pp,
         .lexer = ctx->lexer,
         .ovnacts = &nested,
-        .prereqs = NULL
+        .prereqs = NULL,
     };
-    while (!lexer_match(ctx->lexer, LEX_T_RCURLY)) {
-        if (!parse_action(&inner_ctx)) {
-            break;
-        }
-    }
+    parse_actions(&inner_ctx, LEX_T_RCURLY);
 
-    /* XXX Not really sure what we should do with prerequisites for nested
-     * actions. */
-    expr_destroy(inner_ctx.prereqs);
+    if (prereq) {
+        /* XXX Not really sure what we should do with prerequisites for "arp"
+         * and "nd_na" actions. */
+        expr_destroy(inner_ctx.prereqs);
+        add_prerequisite(ctx, prereq);
+    } else {
+        /* For "clone", the inner prerequisites should just add to the outer
+         * ones. */
+        ctx->prereqs = expr_combine(EXPR_T_AND,
+                                    inner_ctx.prereqs, ctx->prereqs);
+    }
 
     if (inner_ctx.lexer->error) {
         ovnacts_free(nested.data, nested.size);
@@ -1055,9 +1112,8 @@ parse_nested_action(struct action_context *ctx, enum ovnact_type type,
         return;
     }
 
-    add_prerequisite(ctx, prereq);
-
-    struct ovnact_nest *on = ovnact_put(ctx->ovnacts, type, sizeof *on);
+    struct ovnact_nest *on = ovnact_put(ctx->ovnacts, type,
+                                        OVNACT_ALIGN(sizeof *on));
     on->nested_len = nested.size;
     on->nested = ofpbuf_steal_data(&nested);
 }
@@ -1074,6 +1130,12 @@ parse_ND_NA(struct action_context *ctx)
     parse_nested_action(ctx, OVNACT_ND_NA, "nd_ns");
 }
 
+static void
+parse_CLONE(struct action_context *ctx)
+{
+    parse_nested_action(ctx, OVNACT_CLONE, NULL);
+}
+
 static void
 format_nested_action(const struct ovnact_nest *on, const char *name,
                      struct ds *s)
@@ -1096,10 +1158,16 @@ format_ND_NA(const struct ovnact_nest *nest, struct ds *s)
 }
 
 static void
-encode_nested_actions(const struct ovnact_nest *on,
-                      const struct ovnact_encode_params *ep,
-                      enum action_opcode opcode,
-                      struct ofpbuf *ofpacts)
+format_CLONE(const struct ovnact_nest *nest, struct ds *s)
+{
+    format_nested_action(nest, "clone", s);
+}
+
+static void
+encode_nested_neighbor_actions(const struct ovnact_nest *on,
+                               const struct ovnact_encode_params *ep,
+                               enum action_opcode opcode,
+                               struct ofpbuf *ofpacts)
 {
     /* Convert nested actions into ofpacts. */
     uint64_t inner_ofpacts_stub[1024 / 8];
@@ -1124,7 +1192,7 @@ encode_ARP(const struct ovnact_nest *on,
            const struct ovnact_encode_params *ep,
            struct ofpbuf *ofpacts)
 {
-    encode_nested_actions(on, ep, ACTION_OPCODE_ARP, ofpacts);
+    encode_nested_neighbor_actions(on, ep, ACTION_OPCODE_ARP, ofpacts);
 }
 
 static void
@@ -1132,27 +1200,29 @@ encode_ND_NA(const struct ovnact_nest *on,
              const struct ovnact_encode_params *ep,
              struct ofpbuf *ofpacts)
 {
-    encode_nested_actions(on, ep, ACTION_OPCODE_ND_NA, ofpacts);
+    encode_nested_neighbor_actions(on, ep, ACTION_OPCODE_ND_NA, ofpacts);
 }
 
 static void
-free_nested_actions(struct ovnact_nest *on)
+encode_CLONE(const struct ovnact_nest *on,
+             const struct ovnact_encode_params *ep,
+             struct ofpbuf *ofpacts)
+{
+    size_t ofs = ofpacts->size;
+    ofpact_put_CLONE(ofpacts);
+    ovnacts_encode(on->nested, on->nested_len, ep, ofpacts);
+
+    struct ofpact_nest *clone = ofpbuf_at_assert(ofpacts, ofs, sizeof *clone);
+    ofpacts->header = clone;
+    ofpact_finish_CLONE(ofpacts, &clone);
+}
+
+static void
+ovnact_nest_free(struct ovnact_nest *on)
 {
     ovnacts_free(on->nested, on->nested_len);
     free(on->nested);
 }
-
-static void
-free_ARP(struct ovnact_nest *nest)
-{
-    free_nested_actions(nest);
-}
-
-static void
-free_ND_NA(struct ovnact_nest *nest)
-{
-    free_nested_actions(nest);
-}
 
 static void
 parse_get_mac_bind(struct action_context *ctx, int width,
@@ -1223,12 +1293,7 @@ encode_GET_ND(const struct ovnact_get_mac_bind *get_mac,
 }
 
 static void
-free_GET_ARP(struct ovnact_get_mac_bind *get_mac OVS_UNUSED)
-{
-}
-
-static void
-free_GET_ND(struct ovnact_get_mac_bind *get_mac OVS_UNUSED)
+ovnact_get_mac_bind_free(struct ovnact_get_mac_bind *get_mac OVS_UNUSED)
 {
 }
 
@@ -1302,12 +1367,7 @@ encode_PUT_ND(const struct ovnact_put_mac_bind *put_mac,
 }
 
 static void
-free_PUT_ARP(struct ovnact_put_mac_bind *put_mac OVS_UNUSED)
-{
-}
-
-static void
-free_PUT_ND(struct ovnact_put_mac_bind *put_mac OVS_UNUSED)
+ovnact_put_mac_bind_free(struct ovnact_put_mac_bind *put_mac OVS_UNUSED)
 {
 }
 
@@ -1604,23 +1664,11 @@ encode_PUT_DHCPV6_OPTS(const struct ovnact_put_dhcp_opts *pdo,
 }
 
 static void
-free_put_dhcp_opts(struct ovnact_put_dhcp_opts *pdo)
+ovnact_put_dhcp_opts_free(struct ovnact_put_dhcp_opts *pdo)
 {
     free_dhcp_options(pdo->options, pdo->n_options);
 }
 
-static void
-free_PUT_DHCPV4_OPTS(struct ovnact_put_dhcp_opts *pdo)
-{
-    free_put_dhcp_opts(pdo);
-}
-
-static void
-free_PUT_DHCPV6_OPTS(struct ovnact_put_dhcp_opts *pdo)
-{
-    free_put_dhcp_opts(pdo);
-}
-
 static void
 parse_SET_QUEUE(struct action_context *ctx)
 {
@@ -1657,7 +1705,7 @@ encode_SET_QUEUE(const struct ovnact_set_queue *set_queue,
 }
 
 static void
-free_SET_QUEUE(struct ovnact_set_queue *a OVS_UNUSED)
+ovnact_set_queue_free(struct ovnact_set_queue *a OVS_UNUSED)
 {
 }
 
@@ -1719,6 +1767,10 @@ parse_action(struct action_context *ctx)
         parse_CT_SNAT(ctx);
     } else if (lexer_match_id(ctx->lexer, "ct_lb")) {
         parse_ct_lb_action(ctx);
+    } else if (lexer_match_id(ctx->lexer, "ct_clear")) {
+        ovnact_put_CT_CLEAR(ctx->ovnacts);
+    } else if (lexer_match_id(ctx->lexer, "clone")) {
+        parse_CLONE(ctx);
     } else if (lexer_match_id(ctx->lexer, "arp")) {
         parse_ARP(ctx);
     } else if (lexer_match_id(ctx->lexer, "nd_na")) {
@@ -1741,7 +1793,7 @@ parse_action(struct action_context *ctx)
 }
 
 static void
-parse_actions(struct action_context *ctx)
+parse_actions(struct action_context *ctx, enum lex_type sentinel)
 {
     /* "drop;" by itself is a valid (empty) set of actions, but it can't be
      * combined with other actions because that doesn't make sense. */
@@ -1750,11 +1802,11 @@ parse_actions(struct action_context *ctx)
         && lexer_lookahead(ctx->lexer) == LEX_T_SEMICOLON) {
         lexer_get(ctx->lexer);  /* Skip "drop". */
         lexer_get(ctx->lexer);  /* Skip ";". */
-        lexer_force_end(ctx->lexer);
+        lexer_force_match(ctx->lexer, sentinel);
         return;
     }
 
-    while (ctx->lexer->token.type != LEX_T_END) {
+    while (!lexer_match(ctx->lexer, sentinel)) {
         if (!parse_action(ctx)) {
             return;
         }
@@ -1789,7 +1841,7 @@ ovnacts_parse(struct lexer *lexer, const struct ovnact_parse_params *pp,
         .prereqs = NULL,
     };
     if (!lexer->error) {
-        parse_actions(&ctx);
+        parse_actions(&ctx, LEX_T_END);
     }
 
     if (!lexer->error) {
@@ -1903,7 +1955,7 @@ ovnact_free(struct ovnact *a)
     switch (a->type) {
 #define OVNACT(ENUM, STRUCT)                                            \
         case OVNACT_##ENUM:                                             \
-            free_##ENUM(ALIGNED_CAST(struct STRUCT *, a));              \
+            STRUCT##_free(ALIGNED_CAST(struct STRUCT *, a));            \
             break;
         OVNACTS
 #undef OVNACT
