@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2009, 2010, 2013, 2014, 2016, 2017 Nicira, Inc.
+ * Copyright (c) 2008, 2009, 2010, 2013, 2014, 2016 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -412,6 +412,7 @@ find_batch(const struct cmap *cmap, const int value)
 {
     size_t i, ret;
     const size_t end = MIN(n_batch, n_elems - value);
+    unsigned long map = ~0;
     uint32_t hashes[N_BATCH_MAX];
     const struct cmap_node *nodes[N_BATCH_MAX];
 
@@ -430,7 +431,7 @@ find_batch(const struct cmap *cmap, const int value)
 
     ret = i;
 
-    unsigned long map = i ? ~0UL >> (BITMAP_ULONG_BITS - i) : 0;
+    map >>= BITMAP_ULONG_BITS - i; /* Clear excess bits. */
     map = cmap_find_batch(cmap, map, hashes, nodes);
 
     ULLONG_FOR_EACH_1(i, map) {
